/*
Snell-Descartes

Copyright (C) 2012 Observatoire de Paris-Meudon
 
Auteur : Benjamin Mollier

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

import simu.*;

/**
 * Algorithme de simulation pour SimuApplet.
 * Simule les rayons rflechis et rfracts, issus d'un rayon incident sur un dioptre plan.
 */
public class SnellDescartes extends SimuApplet 
{

    //----- I. Variable globales ------------------------------------------------------------------------------------------------------------------
        // - - - I.1 Coordonnes des rayons incidents et rflchis (x1,y1) et rfract (x2,y2) - - - -
            double x1,y1;
            //double x2,y2;
            double s;
            int f;
    
        // - - - I.2 Pas pour tracs les rayons - - - 
            double ds = 0.01;
    
        // - - - I.3 Angles rflchi et rfract - - -
            double r1,i2;
            double i1;
            double n1,n2;
    
        // - - - I.4 Options graphiques - - -
            String s1,s2;
    
    
    
                
    //----- II. Constructeur de la classe, avec la dfinition des paramtres de simulation et des options d'affichage. ----------------------------
    public SnellDescartes() 
    {
        appletDoc("snell_descartes_doc.xml");
    }
    
    //----- III. Calcul des rayons ----------------------------------------------------------------------------------------------------------------
        // - - - III.1 Calcul live (utilis avec affichage "plotlive") - - - - 
            // - 3.1.1 Initialisation -
                public void initCalculLive(ListeValeurs in) throws SimuException 
                {
                    // - 3.1.1.1 Chargement des variables d'entre -
                    i1 = in.lireDouble("i1");
                    n1 = (Math.abs(i1)<=90 ? in.lireDouble("n1") : in.lireDouble("n2"));
                    n2 = (Math.abs(i1)<=90 ? in.lireDouble("n2") : in.lireDouble("n1"));
                    s1 = in.lireString("s1");
                    s2 = in.lireString("s2");
                    
                    
                    // - 3.1.1.2 Initialisation des coordonnes -
                    s  =  8;
                    x1 = -s * Math.sin(i1/180.*Math.PI);
                    y1 =  s * Math.cos(i1/180.*Math.PI);
        
                    // - 3.1.1.3 Calcul des angles rflchi et rfract
                    r1 = -i1;
                    i2 = 180/Math.PI * Math.asin(n1/n2*Math.sin(i1/180*Math.PI));
                    
                    
                    
                }
            // - 3.1.2 Calcul de l'abscisse curviligne s  s+ds
                public ListeValeurs calculLive() throws SimuException 
                {
                    ListeValeurs out = new ListeValeurs();
                    
                    if (s > 0)
                    {
                        s -= ds;
                        x1 = -s * Math.sin(i1/180.*Math.PI);
                        y1 =  s * Math.cos(i1/180.*Math.PI);
                        f = 0;
                    }
                    else if (s<=0 && s>-8)
                    {
                        
                        if (f==0)
                        {
                            s -=  ds;
                            x1 =  s * Math.sin(r1/180.*Math.PI);
                            y1 = -s * Math.cos(r1/180.*Math.PI);
                            f = 1;
                        }
                        else if (f==1)
                        {
                            s -=  ds;
                            x1 = -s * Math.sin(i2/180.*Math.PI);
                            y1 =  s * Math.cos(i2/180.*Math.PI)*(Math.abs(i1)<=90 ? 1. : -1.);
                            f = 0;
                        }
                        
                        
                    }
                    else
                        return null; 
        
       
                    out.ajouterDouble("x1", x1);
                    out.ajouterDouble("y1", y1);
                    out.ajouterDouble("i1", i1);
                    out.ajouterDouble("r1", r1);
                    out.ajouterDouble("i2", i2);
                    out.ajouterDouble("i1b", ("on".equals(s1)?Math.sin(i1/180*Math.PI):i1));
                    out.ajouterDouble("i2b" , ("on".equals(s2)?Math.sin(i2/180*Math.PI):i2));
                    
                    dormir(Math.round(ds*100)); // pour bloquer le programme pendant ds secondes
                    
                    return out;
                }
    
    // - - - III.2 Calcul complet d'un coup (utilis avec les affichages "plot" et "tableau") - - - - 
    public ListeValeurs calcul(ListeValeurs in) throws SimuException 
    {
        ListeValeurs out = new ListeValeurs();
        
        // - 3.1.1.1 Chargement des variables d'entre -
        n1 = in.lireDouble("n1");
        n2 = in.lireDouble("n2");
        s1 = in.lireString("s1");
        s2 = in.lireString("s2");
        
        
        for (i1 = 0; i1 <= 90 ; i1 = i1 + 0.1)
        {
            r1 = -i1;
            i2 = 180/Math.PI * Math.asin(n1/n2*Math.sin(i1/180*Math.PI));
            
            out.ajouterDouble("x1", x1);
            out.ajouterDouble("y1", y1);
            out.ajouterDouble("i1", i1);
            out.ajouterDouble("i2", i1);
            out.ajouterDouble("r1", r1);
            out.ajouterDouble("i1b", ("on".equals(s1)?Math.sin(i1/180*Math.PI):i1));
            out.ajouterDouble("i2b" , ("on".equals(s2)?Math.sin(i2/180*Math.PI):i2));
        }
        return out;
    }
}
